det<-function(m)
  {
# computes the determinant of a symmatric matrix
  erg<-prod((eigen(m,symmetric=T,only.values=T))$values)
  erg
  }

logdet<-function(m)
  {
#computes the logarithm of the determinant of a symmetric matrix
#numerical superior to log(det(m))
  erg<-sum(log(eigen(m,symmetric=T,only.values=T)$values))
  erg
  }

addtodiag<-function(m, d, jstart=1, jend=length(d))
  {
#adds the elements of d to the diagonal of m
  for(j in jstart:jend)
    {
    m[j,j]<-m[j,j]+d[j-jstart+1]
    }
  m
  }

addtodiag.c<-function(x,w,jstart=1,jend=length(w))
  {
#adds the elements of w to the diagonal of x
  matrix(.C("addtodiag",as.double(x),as.double(w),as.integer(jstart),as.integer(jend),dim(x)[[1]])[[1]],dim(x)[[1]],dim(x)[[2]])
  }

spur<-function(m)
  {
# computes the trace of m
  erg <- sum(diag(m))
  erg
  }

spur2<-function(m1,m2)
  {
#computes the trace of the product of two (symmetric) matrices m1 and m2
  erg<-sum(m1*m2)
  erg
  }

makeLambdainv<-function(theta,bcut)
  {
#computes a vector containing the inverse diagonal elements of the covariance matrix of the random effects
  erg<-rep(1/theta[1],bcut[2]-1)
  if(length(theta)>1)
    {
    for(j in 2:length(theta))
      {
      erg<-c(erg,rep(1/theta[j],bcut[j+1]-bcut[j]))
      }
    }
  erg
  }

makeLambdainv2<-function(theta,bcut)
  {
# computes the inverse covariance matrix of the random effects (in absence of a dispersion parameter)
  erg<-rep(1/theta[1],bcut[2]-1)
  if(length(bcut)>2)
    {
    for(j in 2:(length(bcut)-1))
      {
      erg<-c(erg,rep(1/theta[j],bcut[j+1]-bcut[j]))
      }
    }
  erg<-diag(erg)
  erg
  }

makeLambda<-function(theta,bcut)
  {
#computes a vector containing the diagonal elements of the covariance matrix of the random effects
  erg<-rep(theta[1],bcut[2]-1)
  if(length(theta)>1)
    {
    for(j in 2:length(theta))
      {
      erg<-c(erg,rep(theta[j],bcut[j+1]-bcut[j]))
      }
    }
  erg
  }

center<-function(z)
  {
#centers the vector z
  erg<-z-mean(z)
  erg
  }

incident<-function(a,b)
  {
# returns incident matrix for observed regions stored in a and possible regions stored in b
  erg<-matrix(0,length(a),length(b))
  for(j in 1:length(b))
    {
    erg[a==b[j],j]<-1
    }
  erg
  }

incident2<-function(a)
  {
# returns incident matrix for observed group-memberships stored in a (distinct levels are computed and stored in randoms)
  h1<-sort(a)
  h2<-c(Inf,h1[1:(length(a)-1)])
  h<-h1[h1[]!=h2[]]
  return(list(incident=incident(a,h),randoms=h))
  }

multdiagfront<-function(m,w)
  {
#performs the matrix multiplication W*x with W=diag(w)
  for(i in 1:length(w))
    {
    m[i,]<-m[i,]*w[i]
    }
  m
  }

multdiagback<-function(m,w)
  {
#performs the matrix multiplication x*W with W=diag(w)
  for(i in 1:length(w))
    {
    m[,i]<-m[,i]*w[i]
    }
  m
  }

plotsort1<-function(m1,m2,m3,m4,m5,m6,m7,j)
  {
#plots estimate of the smooth effect fj (m2), together with frequentist (m3,m4) and bayesian (m5,m6) confidence interval
#m7 contains the linear part of fj
#m1 contains the distinct (ordered) values of xj
  minimum<-min(m5)
  maximum<-max(m6)
  plot(m1,m2,type="l",ylim=c(minimum,maximum),xlab=paste("x",j,sep=""),ylab=paste("f",j,sep=""))
  lines(m1,m3,type="l",lty=2)
  lines(m1,m4,type="l",lty=2)
  lines(m1,m5,type="l",lty=3)
  lines(m1,m6,type="l",lty=3)
  lines(m1,m7,type="l",lty=4)
  }

plotsort<-function(m1,m2,m3,m4,m5,m6,j)
  {
#plots estimate of the smooth effect fj (m2), together with frequentist (m3,m4) and bayesian (m5,m6) confidence interval
#m1 contains the distinct (ordered) values of xj
  minimum<-min(m5)
  maximum<-max(m6)
  plot(m1,m2,type="l",ylim=c(minimum,maximum),xlab=paste("x",j,sep=""),ylab=paste("f",j,sep=""))
  lines(m1,m3,type="l",lty=2)
  lines(m1,m4,type="l",lty=2)
  lines(m1,m5,type="l",lty=3)
  lines(m1,m6,type="l",lty=3)
  }

plotf<-function(sm,include.lin=F)
  {
#plots all nonparametric function estimates stored in the ggamm-object sm
  q<-length(sm)-4
  for(j in 1:q)
    {
    if(names(sm)[j+1]==paste("smoothedEffects",j,sep=""))
      {
      if(include.lin==F)
        {
        plotsort((sm[[1+j]])[,1],(sm[[1+j]])[,2],(sm[[1+j]])[,3],(sm[[1+j]])[,4],(sm[[1+j]])[,7],(sm[[1+j]])[,8],j)
        }
      else
        {
        plotsort1((sm[[1+j]])[,1],(sm[[1+j]])[,2],(sm[[1+j]])[,3],(sm[[1+j]])[,4],(sm[[1+j]])[,7],(sm[[1+j]])[,8],(sm[[1+j]])[,11],j)
        }
      }
    }
  }

chi.mix<-function(df=c(0,1),weight=c(0.5,0.5),arg=1)
  {
# returns the probability P(Q>arg) where Q is a mixture of chi^2-distributed random variables
# with degrees of freedom 'df' and mixing weights 'weight'
  o<-order(df)
  df<-df[o]
  weight<-weight[o]
  if(df[1]!=0)
    {
    erg<-as.vector(matrix(pchisq(rep(arg,length(df)),rep(df,each=length(arg))),nrow=length(arg),ncol=length(df))%*%weight)
    }
  else
    {
    erg<-sum(weight[df==0])+as.vector(matrix(pchisq(rep(arg,length(df[df>0])),rep(df[df>0],each=length(arg))),nrow=length(arg),ncol=length(df[df>0]))%*%weight[df>0])
    }
  erg[arg<0]<-1
  return(1-erg)
  }

plotmap<-function(sm,map)
  {
#plots the spatial effect stored in the ggamm-object sm
  drawmap(map=map,regionvar=sm$spatialEffects[,1],plotvar=sm$spatialEffects[,2],color=T,legend=T)
  }

drawmap<-
function(map, dfile, outfile, regionvar, plotvar, lowerlimit, upperlimit, 
	nrcolors = 100, pstitle = "", color = F, legend = T, drawnames = F, 
	swapcolors = F, pcat = F)
{
	if(!missing(outfile) && !missing(dfile)) {
		if(dfile == outfile)
			stop("Argument \"dfile\" and argument \"outfile\" are identical"
				)
	}
	if(pcat)
		nrcolors <- 3
	black <- 1
	xmin <- 1:length(map)
	xmax <- 1:length(map)
	ymin <- 1:length(map)
	ymax <- 1:length(map)
	for(i in 1:length(map)) {
		xmin[i] <- min(map[[i]][, 1], na.rm = T)
		xmax[i] <- max(map[[i]][, 1], na.rm = T)
		ymin[i] <- min(map[[i]][, 2], na.rm = T)
		ymax[i] <- max(map[[i]][, 2], na.rm = T)
	}
	xlimits <- c(min(xmin), max(xmax))
	ylimits <- c(min(ymin) - (max(ymax) - min(ymin)) * 0.1, max(ymax))
	ratio.xy <- diff(range(ylimits))/diff(range(xlimits))
	if(missing(dfile) && missing(regionvar) && missing(plotvar)) {
# draw only borders
		if(missing(outfile)) win.graph() else postscript(outfile, 
				horizontal = F, colors = 0, width = 8, height
				 = 9.5 * ratio.xy)
		plot(xlimits, ylimits, type = "n", axes = F, col = 0, xlab = "",
			ylab = "")
		for(k in 1:length(map))
			polygon(map[[k]][, 1], map[[k]][, 2], density = 0, lwd
				 = 0.3, col = 1)
	}
	else {
# draw and fill borders
		if(missing(dfile)) {
			ord <- order(regionvar)
			plotvar <- plotvar[ord]
			regionvar <- regionvar[ord]
		}
		else {
			data <- read.table(dfile, header = T, row.names = NULL)
			ord <- order(data[, regionvar])
			plotvar <- data[, plotvar]
			plotvar <- plotvar[ord]
			regionvar <- data[, regionvar]
			regionvar <- regionvar[ord]
		}
		if(!missing(outfile)) {
# define colors
			black <- nrcolors + 1
			white <- nrcolors + 2
			if(color == T) {
				ps.colors <- matrix(1, nrow = white, ncol = 3)
				if(swapcolors == F)
				  ps.colors[1:nrcolors, 1] <- ((1:nrcolors) - 1
				    )/(3 * (nrcolors - 1))
				else ps.colors[1:nrcolors, 1] <- ((nrcolors:1) - 
				    1)/(3 * (nrcolors - 1))
				ps.colors[black,  ] <- c(0, 0, 0)
				ps.colors[white,  ] <- c(0, 0, 1)
			}
			else {
				if(swapcolors == F)
				  ps.colors <- ((1:nrcolors) - 1)/(nrcolors - 1
				    )
				else ps.colors <- ((nrcolors:1) - 1)/(nrcolors - 
				    1)
				ps.colors[black] <- 0
				ps.colors[white] <- 1
			}
			postscript(outfile, horizontal = F, colors = ps.colors, 
				width = 8, height = 9.5 * ratio.xy)
			legend.colors <- 1:nrcolors
		}
		else {
			screen.colors <- as.integer(seq(0, 255, length = 2))
			screen.colors <- matrix(screen.colors, ncol = 3, nrow
				 = 2)
			black <- 1
			white <- 2
			if(color == T) {
				if(nrcolors %% 2 == 1) {
				  if(swapcolors == F)
				    graphsheet(color.table = screen.colors, 
				      num.image.colors = 3, num.image.shades = 
				      paste((nrcolors - 3)/2, ",", (nrcolors - 
				      3)/2), image.color.table = 
				      "255,0,0|255,255,0|0,255,0")
				  else graphsheet(color.table = screen.colors, 
				      num.image.colors = 3, num.image.shades = 
				      paste((nrcolors - 3)/2, ",", (nrcolors - 
				      3)/2), image.color.table = 
				      "0,255,0|255,255,0|255,0,0")
				}
				else {
				  shades <- paste(nrcolors/2 - 2, ",0,", 
				    nrcolors/2 - 2)
				  if(swapcolors == F)
				    colors <- paste("255,0,0|255,", as.integer((
				      nrcolors - 1)/nrcolors * 255), ",0|", 
				      as.integer((nrcolors - 1)/nrcolors * 255),
				      ",255,0|0,255,0", sep = "")
				  else colors <- paste("0,255,0|", as.integer((
				      nrcolors - 1)/nrcolors * 255), 
				      ",255,0|255,", as.integer((nrcolors - 1)/
				      nrcolors * 255), ",0|255,0,0", sep = "")
				  graphsheet(color.table = screen.colors, 
				    num.image.colors = 4, num.image.shades = 
				    shades, image.color.table = colors)
				}
			}
			else {
				if(swapcolors == F)
				  graphsheet(color.table = screen.colors, 
				    num.image.colors = 2, num.image.shades = 
				    paste(nrcolors - 2), image.color.table = 
				    "0,0,0|255,255,255")
				else graphsheet(color.table = screen.colors, 
				    num.image.colors = 2, num.image.shades = 
				    paste(nrcolors - 2), image.color.table = 
				    "255,255,255|0,0,0")
			}
			legend.colors <- 1:nrcolors + 16
		}
		maxim <- max(plotvar, na.rm = T)
		minim <- min(plotvar, na.rm = T)
		if(pcat) {
			upperlimit <- 1
			lowerlimit <- -1
		}
		else {
			if(missing(lowerlimit)) {
				lowerlimit <- minim
			}
			else if(lowerlimit > minim) {
				plotvar[plotvar < lowerlimit] <- lowerlimit
				cat(paste(
				  "Note: lowerlimit is above minimum value (", 
				  lowerlimit, " > ", minim, ")\n"))
			}
			if(missing(upperlimit)) {
				upperlimit <- maxim
			}
			else if(upperlimit < maxim) {
				plotvar[plotvar > upperlimit] <- upperlimit
				cat(paste(
				  "Note: upperlimit is below maximum value (", 
				  upperlimit, " < ", maxim, ")\n"))
			}
		}
		fill.colors <- cut(c(lowerlimit, plotvar, upperlimit), nrcolors
			)
		fill.colors <- fill.colors[c(-1,  - length(fill.colors))]
		if(missing(outfile))
			fill.colors <- fill.colors + 16
		plot(xlimits, ylimits, type = "n", axes = F, col = white, xlab
			 = "", ylab = "")
		if(sum(!is.na(match(names(map), regionvar))) == 0)
			warning("map probably doesn't match datafile")
		block1 <- c()
		block2 <- c()
		for(k in 1:length(map)) {
			if(map[[k]][1, 1] == "NA" && map[[k]][1, 2] == "NA")
				block2 <- c(block2, k)
			else block1 <- c(block1, k)
		}
		m <- match(names(map), regionvar)
		for(k in block1) {
			if(is.na(m[k])) {
				polygon(map[[k]][, 1], map[[k]][, 2], col = 
				  white, border = F)
				polygon(map[[k]][, 1], map[[k]][, 2], density
				   = 15, lwd = 0.3, col = black)
			}
			else {
				polygon(map[[k]][, 1], map[[k]][, 2], col = 
				  fill.colors[m[k]], border = F)
				polygon(map[[k]][, 1], map[[k]][, 2], col = 
				  black, density = 0)
			}
		}
		for(k in block2) {
			if(is.na(m[k])) {
				polygon(map[[k]][-1, 1], map[[k]][-1, 2], col
				   = white, border = F)
				polygon(map[[k]][-1, 1], map[[k]][-1, 2], 
				  density = 15, lwd = 0.3, col = black)
			}
			else {
				polygon(map[[k]][-1, 1], map[[k]][-1, 2], col
				   = fill.colors[m[k]], border = F)
				polygon(map[[k]][-1, 1], map[[k]][-1, 2], col
				   = black, density = 0)
			}
		}
		if(legend == T) {
			ylo <- yro <- ylimits[1] + (0.7 * (ylimits[2] - ylimits[
				1]))/11
			ylu <- yru <- ylimits[1] + (0.3 * (ylimits[2] - ylimits[
				1]))/11
			tylu <- tyru <- ylimits[1]
			xlu <- xlo <- xlimits[1]
			xru <- xro <- xlimits[1] + 0.3 * (xlimits[2] - xlimits[
				1])
			step <- (xru - xlu)/nrcolors
			for(i in 0:(nrcolors - 1)) {
				polygon(c(xlo + step * i, xlo + step * (i + 1), 
				  xlu + step * (i + 1), xlu + step * i), c(ylo, 
				  yro, yru, ylu), col = legend.colors[i + 1], 
				  border = F)
			}
			polygon(c(xlo, xro, xru, xlu), c(ylo, yro, yru, ylu), 
				density = 0, col = black, lwd = 0.3)
			text(xlu + 0.5 * step, tylu, lowerlimit, cex = 0.7, col
				 = black)
			text(xru - 0.5 * step, tyru, upperlimit, cex = 0.7, col
				 = black)
			if(lowerlimit + (upperlimit - lowerlimit)/3 < 0 && 0 < 
				upperlimit - (upperlimit - lowerlimit)/3) {
				help <- cut(c(0, lowerlimit, upperlimit), 
				  nrcolors)
				text(xlu + step * (help[1] - 0.5), tylu, "0", 
				  cex = 0.7, col = black)
			}
		}
	}
	if(drawnames == T) {
		xpos <- (xmin + xmax)/2
		ypos <- (ymin + ymax)/2
		text(xpos, ypos, labels = names(map), cex = 0.7, col = black)
	}
	title(main = pstitle, col = black)
	if(!missing(outfile))
		dev.off()
	return(invisible())
}

readbndfile<-
function(path, name)
{
	options(warn = -1)
	data.raw <- scan(path, what = list("", ""), sep = ",")
	data.numeric <- list(as.numeric(data.raw[[1]]), as.numeric(data.raw[[2]]))
	anzkreise <- sum(is.na(data.numeric[[1]])) - sum(data.raw[[1]] == "is.in")
	cat("Note: map consists of", anzkreise, "polygons\n")
	cat("Reading map ...\n")
	map <- list()
	i <- 1
	for(k in 1:anzkreise) {
		j <- 1
		npoints <- data.numeric[[2]][i]
		if(data.numeric[[1]][i + 1] == "NA" && data.numeric[[2]][i + 1] == "NA") {
			npoints <- npoints + 1
		}
		elem1 <- c()
		elem2 <- c()
		for(j in 1:npoints) {
			elem1 <- c(elem1, data.numeric[[1]][i + j])
			elem2 <- c(elem2, data.numeric[[2]][i + j])
		}
		map[[k]] <- matrix(c(elem1, elem2), ncol = 2)
		names(map)[k] <- substring(data.raw[[1]][i], 2, nchar(data.raw[[1]][i]) - 1)
		i <- i + npoints + 1
	}
	if(sum(is.na(as.numeric(names(map)))) == 0) {
		map <- map[order(as.numeric(names(map)))]
		cat("Note: regions sorted by number\n")
	}
	else {
		map <- map[order(names(map))]
		cat("Note: regions sorted by name\n")
	}
	count <- 1
	if(length(map) > 1) {
		for(i in 2:anzkreise) {
			if(names(map)[i] != names(map)[i - 1])
				count <- count + 1
		}
	}
	cat("Note: map consists of", count, "regions\n")
	assign(name, map, where = 1)
	options(warn = 1)
	return(invisible())
}

